'                          (c) 1995-2009, MCS
'                        Bootloader.bas
'  This sample demonstrates how you can write your own bootloader
'  in BASCOM BASIC
'  VERSION 2 of the BOOTLOADER. The waiting for the NAK is stretched
'  further a bug was resolved for the M64/M128 that have a big page size
'-----------------------------------------------------------------
'This sample will be extended to support other chips with bootloader
'The loader is supported from the IDE

'Bootloader changed for ATtiny85 by Burkhard Kainka 2015
'Software UART at 9600 Baud at 8 MHz
'$timeout is not possible,special timeout before Testfor123
'Changes first rjmp in loaded file to rjmp &0D00, Bootloader
'rjmp to start address written to $F80


$regfile = "attiny85.dat"
$crystal = 8000000
$hwstack = 8
$swstack = 4
$framesize = 4

$loader = $c00                                              'this address you can find in the datasheet
Const Maxwordbit = 5
Const Maxword =(2 ^ Maxwordbit) * 2                         '128
Const Maxwordshift = Maxwordbit + 1
Const Cdebug = 0                                            ' leave this to 0

Open "comb.2:9600,8,n,1" For Input As #1
Open "comb.1:9600,8,n,1," For Output As #2

Dim Bstatus As Byte , Bretries As Byte , Bblock As Byte , Bblocklocal As Byte
Dim Bcsum1 As Byte , Bcsum2 As Byte , Buf(128) As Byte , Csum As Byte
Dim J As Byte , Spmcrval As Byte
Dim Dat As Byte                                             ' self program command byte value

Dim Z As Long                                               'this is the Z pointer word
Dim Vl As Byte , Vh As Byte                                 ' these bytes are used for the data values
Dim Wrd As Word , Page As Word                              'these vars contain the page and word address
Dim Bkind As Byte , Bstarted As Byte
Dim Timeouts As Word                                        'used instead of $timeout
Dim Waddress As Word                                        'Address of start

Disable Interrupts                                          'we do not use ints

'some constants used in serial com
Const Nak = &H15
Const Ack = &H06
Const Can = &H18

Do
  Timeouts = Timeouts + 1
  If Timeouts > 60000 Then Goto Do_start
Loop Until Pinb.2 = 0

Bretries = 5                                                'we try 5 times
Testfor123:
Get #1 , Bstatus                                            '1, Waitkey()                                         'wait for the loader to send a byte
Put #2 , Bstatus

If Bstatus = 123 Then                                       'did we received value 123 ?
   Bkind = 0                                                'normal flash loader
   Goto Loader
Elseif Bstatus = 124 Then                                   ' EEPROM
   Bkind = 1                                                ' EEPROM loader
   Goto Loader
Elseif Bstatus <> 0 Then
   Decr Bretries
   If Bretries <> 0 Then Goto Testfor123                    'we test again
End If

Goto Do_start                                               'goto the normal reset vector at address 0


'this is the loader routine. It is a Xmodem-checksum reception routine
Loader:
  If Bkind = 0 Then
     Spmcrval = 3 : Gosub Do_spm                            ' erase  the first page
     Spmcrval = 17 : Gosub Do_spm                           ' re-enable page
  End If

Bretries = 10                                               'number of retries

Do
  Bstarted = 0                                              ' we were not started yet
  Csum = 0                                                  'checksum is 0 when we start
  Put #2 , Nak                                              ' firt time send a nack
  Do

    Get #1 , Bstatus                                        'wait for statuse byte

    Select Case Bstatus
       Case 1:                                              ' start of heading, PC is ready to send
            Incr Bblocklocal                                'increase local block count
            Csum = 1                                        'checksum is 1
            Get #1 , Bblock : Csum = Csum + Bblock          'get block
            Get #1 , Bcsum1 : Csum = Csum + Bcsum1          'get checksum first byte
            For J = 1 To 128                                'get 128 bytes
              Get #1 , Dat                                  'get 128 bytes
              Buf(j) = Dat : Csum = Csum + Buf(j)
              'Toggle Portb.4
            Next
            Get #1 , Bcsum2                                 'get second checksum byte
            If Bblocklocal = Bblock Then                    'are the blocks the same?
               If Bcsum2 = Csum Then                        'is the checksum the same?
                  If Bblock = 1 Then
                    Waddress = Buf(2) - $c0
                    Waddress = Waddress * 256
                    Waddress = Waddress + Buf(1)
                    Waddress = Waddress + 1
                    Buf(1) = $0ff                           'rjmp $0D00 at 0000
                    Buf(2) = $0cb
                  End If
                  Gosub Writepage                           'yes go write the page
                  Put #2 , Ack                              'acknowledge
               Else                                         'no match so send nak
                  Put #2 , Nak
               End If
            Else
               Put #2 , Nak                                 'blocks do not match
            End If
       Case 4:                                              ' end of transmission , file is transmitted
             If Wrd > 0 And Bkind = 0 Then                  'if there was something left in the page
                 Wrd = 0                                    'Z pointer needs wrd to be 0
                 Spmcrval = 5 : Gosub Do_spm                'write page
                 Spmcrval = 17 : Gosub Do_spm               ' re-enable page
             End If
            '  Waitms 100                                   ' OPTIONAL REMARK THIS IF THE DTR SIGNAL ARRIVES TO EARLY

           '  Portb.3 = 0                                    ' simple indication that we are finished and ok
             Waitms 20

             For J = 1 To 128                               'get 128 bytes
               Buf(j) = 255
             Next
             Waddress = $1000 + Waddress                    'rjmp ... in F80
             Waddress = Waddress - $0f81
             Buf(1) = Waddress And 255
             Waddress = Waddress / 256
             Buf(2) = Waddress + $c0
             Wrd = 0
             Page = 124                                     '32 Word * 124 = $0F80
             Spmcrval = 3 : Gosub Do_spm                    ' erase  page
             Spmcrval = 17 : Gosub Do_spm                   ' re-enable page
             Gosub Writepage                                'yes go write the page
             Waitms 20                                      'wait for SPM complete
             Put #2 , Ack                                   ' send ack and ready
             Goto Do_start                                  ' start new program
       Case &H18:                                           ' PC aborts transmission
             Goto Do_start                                  ' ready
       Case 123 : Exit Do                                   'was probably still in the buffer
       Case 124 : Exit Do
       Case Else
          Exit Do                                           ' no valid data
    End Select
  Loop
  If Bretries > 0 Then                                      'attempte left?
     Waitms 1000
     Decr Bretries                                          'decrease attempts
  Else
     Goto Do_start                                          'reset chip
  End If
Loop

'write one or more pages
Writepage:
 If Bkind = 0 Then
   For J = 1 To 128 Step 2                                  'we write 2 bytes into a page
      Vl = Buf(j) : Vh = Buf(j + 1)                         'get Low and High bytes
      lds r0, {vl}                                          'store them into r0 and r1 registers
      lds r1, {vh}
      Spmcrval = 1 : Gosub Do_spm                           'write value into page at word address
      Wrd = Wrd + 2                                         ' word address increases with 2 because LS bit of Z is not used
      If Wrd = Maxword Then                                 ' page is full
          Wrd = 0                                           'Z pointer needs wrd to be 0
          Spmcrval = 5 : Gosub Do_spm                       'write page
          Spmcrval = 17 : Gosub Do_spm                      ' re-enable page

          Page = Page + 1                                   'next page
          Spmcrval = 3 : Gosub Do_spm                       ' erase  next page
          Spmcrval = 17 : Gosub Do_spm                      ' re-enable page
      End If
   Next

 Else                                                       'eeprom
     For J = 1 To 128
       Writeeeprom Buf(j) , Wrd
       Wrd = Wrd + 1
     Next
 End If
Return

Do_spm:
  Bitwait Spmcsr.0 , Reset                                  ' check for previous SPM complete
  Bitwait Eecr.1 , Reset                                    'wait for eeprom

  Z = Page                                                  'make equal to page
  Shift Z , Left , Maxwordshift                             'shift to proper place
  Z = Z + Wrd                                               'add word
  lds r30,{Z}
  lds r31,{Z+1}

  #if _romsize > 65536
      lds r24,{Z+2}
      sts rampz,r24                                         ' we need to set rampz also for the M128
  #endif

  Spmcsr = Spmcrval                                         'assign register
  spm                                                       'this is an asm instruction
  nop
  nop
Return

Do_start:
  Goto $0f80                                                'rjmp ...
